/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <linux/input.h>
#include <linux/ioctl.h>
#include <errno.h>
#include<sys/time.h>
#include "iowarrior.h"


int main (int argc, char **argv) {
    /* for device information */
    struct iowarrior_info info;
    /* A buffer for reading the data, suitable for all IOWarrior-Products */
    unsigned char in_buffer[8];
    /* The file-descriptor of our device */
    int fd = -1;
    /* A result for the operations */
    int result=-1;	//result of read and select operations
    /* the read-set for select */
    fd_set rfds;	
    /* the timeout for select */
    struct timeval tv;	
    int i=0;

    /* we try to open the device attached to /dev/usb/iowarrior0 */
    if(( fd = open( "/dev/usb/iowarrior0", O_RDWR)) < 0 ) {
    	perror( "iowarrior open failed" );
	goto exit;
    }
    /* lets see what product we have */
    if( ioctl( fd, IOW_GETINFO, &info)) {
	perror( "Unable to retrieve device info" );
	goto exit;
    }
    /* print the device information */
    printf("The device attached to /dev/usb/iowarrior0\n");
    printf("VendorId=%04x\n",info.vendor);
    printf("ProductId=%04x ",info.product);
    if(info.product==USB_DEVICE_ID_CODEMERCS_IOW40)
	printf("(IOWarrior40)\n");
    else if(info.product==USB_DEVICE_ID_CODEMERCS_IOW24)
	printf("(IOWarrior24)\n");
    else if(info.product==USB_DEVICE_ID_CODEMERCS_IOW56)
	printf("(IOWarrior56)\n");
    else
	printf("(Ooops, unknown device!)\n");
    printf("Serial=0x%s\nRevision=0x%04x\n",info.serial, info.revision); 
    printf("Speed=");
    if(info.speed==1)
	printf("Low Speed (USB 1.1)\n");
    else if(info.speed==2)
	printf("Full Speed (USB 1.1)\n");
    else if(info.speed==3)
	printf("High Speed (USB 2.0)\n");
    else
	printf("Ooops, speed is unknown!\n");
    printf("Power=%dmA\nPacketSize=%d\n",info.power, info.packet_size); 
    /* checking for the IO-PIN interface */
    if(info.if_num!=0) {
	printf("Opps, this is not the IO-PIN interface\n");
	printf("Can't go on\n");
	goto exit;
    }    
    tv.tv_sec=10;	//timeout is set to 10 seconds 
    tv.tv_usec=0;
    FD_ZERO(&rfds);	//clear the read-descriptors for select
    FD_SET(fd, &rfds);	
    printf("We wait for up to 10 seconds... ");
    result=select(fd+1,&rfds,NULL,NULL,&tv);
    if(result==-1) {
	//error in select
	perror("Error in select");
	goto exit;
    }
    else if(result==0) {
	//select returned 0, no new data to be read
	printf("Timeout in read, so we exit\n");
    }
    else {
	if(FD_ISSET(fd,&rfds)) {
	    memset(&in_buffer,0,sizeof(in_buffer));
	    //We got here, so there should be new data available
	    result = read(fd,in_buffer,info.packet_size);
	    if(result!=info.packet_size) {
		//well thats not what we expected...
		int errcode=errno;
		if(errcode==ENODEV) 
		    printf("Hey, you pulled the plug!\n");
		else
		    perror("Error in reading!");
	    }
	    else {
		//buffer now contains the io-status
		printf("ok, here's the IO-Status:\n");
		for(i=0;i<info.packet_size;i++)
		    printf( "Port%d=%02x\n",i,in_buffer[i]);
	    }
 	}
    }
 exit:
    close( fd );
    exit( 0 );
}
